package org.eclipse.stardust.clustering;

import static org.eclipse.stardust.engine.core.runtime.beans.daemons.DaemonProperties.DAEMON_PERIODICITY_SUFFIX;

import java.util.Date;

import org.eclipse.stardust.common.log.LogManager;
import org.eclipse.stardust.common.log.Logger;
import org.eclipse.stardust.common.config.Parameters;
import org.eclipse.stardust.engine.api.runtime.AdministrationService;
import org.eclipse.stardust.engine.api.runtime.Daemon;

/**
 * Action to examine the health state of a {@link Daemon} and possibly restart
 * it if the daemon appears not to be running.
 * 
 * @author JanHendrik.Scheufen
 */
public class CheckDaemonHealthStateAction implements IDaemonAdministrationAction
{
	private static Logger LOG = LogManager.getLogger(CheckDaemonHealthStateAction.class);
	/**
	 * The tolerance threshold for a daemon's execution time.
	 */
	private long toleranceThreshold;

	public CheckDaemonHealthStateAction(long toleranceThreshold) {
		this.toleranceThreshold = toleranceThreshold;
	}

	public void executeWithDaemon(AdministrationService adminService,
			Daemon daemon, String partition)
	{
		// do some guessing to see whether the daemon had been running
		// rely on startTime > 0 to mean "it was started"
		if ((null != daemon.getStartTime())
				&& (0 < daemon.getStartTime().getTime())) {

			int periodicityInSeconds = getDaemonPeriodicity(daemon);

			Date lastExecution = daemon.getLastExecutionTime();
			Date now = new Date();
			long executionTimeDelta = (now.getTime() - lastExecution.getTime()) - (periodicityInSeconds * 1000);
			
			// the time since the last execution should not exceed the scheduled interval
			if ( executionTimeDelta > 0)
			{
				if( LOG.isDebugEnabled() ) {
					LOG.debug("The last execution time of the '"+daemon.getType()+"' exceeds the scheduled" +
						" periodicity ("+periodicityInSeconds+" sec.) by "+executionTimeDelta+" ms.");
				}
				// However, a tolerance is added since the daemon's recorded last execution
				// time consists of the periodicity + the execution time to run the daemon.
				if( executionTimeDelta > toleranceThreshold ) {
					LOG.info("Daemon's last scheduled execution time exceeds tolerance level of "
							+toleranceThreshold+" ms. Restarting daemon '" + daemon.getType() + "'.");
					adminService.startDaemon(daemon.getType(), false);
				}
			}
		}
	}

	private int getDaemonPeriodicity(Daemon daemon) {
		return Parameters.instance().getInteger(
				daemon.getType() + DAEMON_PERIODICITY_SUFFIX, 3);
	}

}
