/*
 * $Id: $
 * (C) 2000 - 2010 CARNOT AG
 */
package org.eclipse.stardust.clustering;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TimerTask;

import org.eclipse.stardust.common.Action;
import org.eclipse.stardust.common.log.LogManager;
import org.eclipse.stardust.common.log.Logger;
import org.eclipse.stardust.common.config.Parameters;
import org.eclipse.stardust.common.config.PropertyLayer;
import org.eclipse.stardust.engine.api.runtime.AdministrationService;
import org.eclipse.stardust.engine.api.runtime.Daemon;
import org.eclipse.stardust.engine.api.runtime.LoginUtils;
import org.eclipse.stardust.engine.core.runtime.beans.AdministrationServiceImpl;
import org.eclipse.stardust.engine.core.runtime.beans.AuditTrailPartitionBean;
import org.eclipse.stardust.engine.core.runtime.beans.ForkingService;
import org.eclipse.stardust.engine.core.runtime.beans.interceptors.AbstractLoginInterceptor;
import org.eclipse.stardust.engine.core.runtime.beans.interceptors.PropertyLayerProviderInterceptor;
import org.eclipse.stardust.engine.core.runtime.beans.removethis.SecurityProperties;

public class DaemonWatchDog extends TimerTask
{

	private static Logger logger = LogManager.getLogger(DaemonWatchDog.class);

	protected ForkingService forkingService;
	
	private CheckDaemonHealthStateAction checkHealthAction= new CheckDaemonHealthStateAction(5000l);
	private StartupDaemonAction startupAction = new StartupDaemonAction( new HashMap<String,List<String>>() );
	
	private boolean initialized = false;

	@SuppressWarnings("unchecked")
	@Override
	public void run() {

		forkingService.isolate(new Action() {
			public Object execute() {
				logger.debug("Executing Daemon Health Check.");
				executeForAllDaemons( checkHealthAction );
				return null;
			}
		});

	}

	/**
	 * Iterates through all partitions and all daemons to execute the
	 * specified action on the daemon.
	 * 
	 * @param action
	 */
	@SuppressWarnings("unchecked")
	protected void executeForAllDaemons(IDaemonAdministrationAction action) {
		final PropertyLayer layer = PropertyLayerProviderInterceptor
				.getCurrent();

		try {
			AdministrationService adminService = new AdministrationServiceImpl();

			for (Iterator<AuditTrailPartitionBean> _iterator = AuditTrailPartitionBean
					.findAll(); _iterator.hasNext();) {

				AuditTrailPartitionBean partition = _iterator.next();
				setPartition(layer, partition.getId());

				List<Daemon> allDaemons = adminService.getAllDaemons(false);
				for (Daemon daemon : allDaemons) {
					try {
						action.executeWithDaemon(adminService, daemon, partition.getId());
					}
					catch (Exception e) {
						logger.error("Error during execution of action "+action.getClass().getName()+" for daemon "+daemon.getType());
					}
				}
			}
		} catch( Exception e ) {
			logger.error("Unable to execute daemon action "+action.getClass().getName(), e);
		}
	}

	@SuppressWarnings("unchecked")
	private void setPartition(PropertyLayer layer, String partitionId) {
		Map properties = new HashMap();
		properties.put(SecurityProperties.PARTITION, partitionId);
		LoginUtils.mergeDefaultCredentials(Parameters.instance(), properties);
		AbstractLoginInterceptor.setCurrentPartitionAndDomain(Parameters
				.instance(), layer, properties);
	}
	
	@SuppressWarnings("unchecked")
	public void initializeDaemonStartup()
	{
		if( !initialized ) {
			forkingService.isolate(new Action() {
				public Object execute() {
					logger.debug("Executing Automatic Daemon Initialization.");
					executeForAllDaemons( startupAction );
					initialized = true;
					return null;
				}
			});
		}
	}

	/**
	 * Sets the daemons that should be started automatically. The specified map
	 * should contain the ID of a partition as the key and a comma-separated list
	 * of daemon types as the value.<br/>
	 * Known daemon types are: event.daemon, mail.trigger, timer.trigger
	 * 
	 * @param startupConfig
	 */
	public void setDaemonStartupConfiguration(Map<String,String> startupConfig)
	{
		final Map<String,List<String>> daemonMap = new HashMap<String,List<String>>(startupConfig.size());
		for( String partition : startupConfig.keySet() ) {
			String[] daemons = startupConfig.get(partition).split(",");
			List<String> daemonList = new ArrayList<String>(daemons.length);
			for( String d : daemons ) {
				daemonList.add( d.trim() );
			}
			daemonMap.put( partition, daemonList );
					
		}
		this.startupAction = new StartupDaemonAction(daemonMap);
	}

	public void setForkingService(ForkingService forkingService) {
		this.forkingService = forkingService;
	}

	/**
	 * Sets the tolerance threshold for the health state check
	 * @param threshold the tolerance in milliseconds
	 */
	public void setToleranceThreshold( long threshold ) {
		this.checkHealthAction = new CheckDaemonHealthStateAction(threshold);
	}
	

}
